(function (factory) {
    if (typeof module === "object" && typeof module.exports === "object") {
        var v = factory(require, exports);
        if (v !== undefined) module.exports = v;
    }
    else if (typeof define === "function" && define.amd) {
        define(["require", "exports"], factory);
    }
})(function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    class RedBlackTreeNode {
        constructor(value, parent, color) {
            this.color = false;
            this.left = RedBlackTreeNode.sentinel;
            this.right = RedBlackTreeNode.sentinel;
            this.parent = RedBlackTreeNode.sentinel;
            this.value = value;
            this.parent = parent;
            this.color = color;
        }
        hasLeftChild() {
            return this.left !== RedBlackTreeNode.sentinel;
        }
        isBlack() {
            return this.color === true;
        }
        hasRightChild() {
            return this.right !== RedBlackTreeNode.sentinel;
        }
        isLeftChild() {
            return this.parent && this.parent.left === this;
        }
        isRed() {
            return this.color === false;
        }
        isRightChild() {
            return this.parent && this.parent.right === this;
        }
        isSentinel() {
            return this === RedBlackTreeNode.sentinel;
        }
        isNotSentinel() {
            return this !== RedBlackTreeNode.sentinel;
        }
        isRoot() {
            return this.parent === RedBlackTreeNode.sentinel;
        }
        setRed() {
            this.color = false;
        }
        setBlack() {
            this.color = true;
        }
        min() {
            if (this.left !== RedBlackTreeNode.sentinel) {
                return this.left.min();
            }
            return this;
        }
        max() {
            if (this.right !== RedBlackTreeNode.sentinel) {
                return this.right.max();
            }
            return this;
        }
    }
    RedBlackTreeNode.sentinel = new RedBlackTreeNode(null, null, true);
    exports.default = RedBlackTreeNode;
});
//# sourceMappingURL=RedBlackTreeNode.js.map